/*
  lcd.c - LCD functions for the Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    Mar-2009

  Assumes a character mode LCD with HD44780 controller
  CPLD mode (LCD_XIO) uses PXD0 to PXD7, LCD_RS, LCD_RW, LCD_E and LCD_BL
  GPIO mode (no LCD_XIO) uses PC0 to PC7, PF0 to PF1 and PD0 to PD1
  Defining OPT_LCD_PORT uses PG0 to PG1 for data instead of PC0 to PC7
  Defining OPT_LCD_ANSI uses ANSI terminal in COM2 instead of PC0 to PC7
*/

#include "lcd.h"

static void delay_us(unsigned long);

static unsigned long lcdControl;
static unsigned long lcdDisplay;

/* -------------------- LCD functions -------------------- */
/* Initialize HD44780 LCD Controller */
void LcdInit(void)
{
    lcdControl = LCD_CTL_INIT;
    lcdDisplay = LCD_DISP_INIT;

#ifdef LCD_XIO
    XioInit();
    XioWrite(LCD_CONTROL, lcdControl);
#elif defined(OPT_LCD_ANSI)
	AnsiInit(1); /* COM2 (UART1) */
#else
    /* Disable JTAG to be able to use PC0-3 */
    JtagDisable();
    /* Enable data port for output */
    SysCtlPeripheralEnable(LCD_PORT_SYSCTL);
    GPIOPinTypeGPIOOutput(LCD_PORT, LCD_PORT_MASK);
    GPIOPinWrite(LCD_PORT, LCD_PORT_MASK, 0);
    /* Enable control lines for output */
    SysCtlPeripheralEnable(LCD_CONTROL_SYSCTL);
    GPIOPinTypeGPIOOutput(LCD_CONTROL, LCD_CONTROL_MASK);
    GPIOPinWrite(LCD_CONTROL, LCD_CONTROL_MASK, lcdControl);
    SysCtlPeripheralEnable(LCD_CONTROL2_SYSCTL);
    GPIOPinTypeGPIOOutput(LCD_CONTROL2, LCD_CONTROL2_MASK);
    GPIOPinWrite(LCD_CONTROL2, LCD_CONTROL2_MASK, 0x00);
#endif

    LcdCommand(0x38); /* 8-bit bus, 2-line display, 5x7 font */
    delay_us(20000); /* 20 ms delay */
    LcdCommand(0x06); /* cursor increment mode, not shifted */
    LcdDisplay(1); /* turn display on (default cursor off, blink off) */
    LcdClear(); /* clear display */
}

/* Write command byte to LCD */
unsigned long LcdCommand(unsigned long data)
{
#ifdef LCD_XIO
    return LcdWriteByte(data, lcdControl);
#elif defined(OPT_LCD_ANSI)
	return 0;
#else
    return LcdWriteByte(data, 0x00);
#endif
}

/* Write data byte to LCD */
unsigned long LcdWrite(unsigned long data)
{
#ifdef LCD_XIO
    return LcdWriteByte(data, (lcdControl | LCD_RS));
#elif defined(OPT_LCD_ANSI)
    AnsiPutChar((unsigned char)data);
    return 0;
#else
    return LcdWriteByte(data, LCD_RS);
#endif
}

/* Write command byte to LCD */
unsigned long LcdWriteByte(unsigned long data, unsigned long ctl)
{
#ifdef LCD_XIO
    XioWrite(LCD_CONTROL, ctl); /* Use LCD_RS=0 for cmd */
    XioWrite(LCD_PORT, data);  /* Write byte to LCD */
    XioWrite(LCD_CONTROL, (ctl | LCD_E)); /* Pulse enable with LCD_E = 1 */
    XioWrite(LCD_CONTROL, ctl); /* Finalize write using LCD_E=0 */
#elif !defined(OPT_LCD_ANSI)
    GPIOPinWrite(LCD_CONTROL2, LCD_CONTROL2_MASK, ctl);
                               /* Use LCD_RS=0 for cmd */
    GPIOPinWrite(LCD_PORT, LCD_PORT_MASK, data);  /* Write byte to LCD */
    GPIOPinWrite(LCD_CONTROL, LCD_CONTROL_MASK, (lcdControl | LCD_E));
                               /* Pulse enable with LCD_E = 1 */
    GPIOPinWrite(LCD_CONTROL, LCD_CONTROL_MASK, lcdControl);
                               /* Finalize write using LCD_E=0 */
#endif
    delay_us(100); /* 100 us delay */
    return 0;
}

/* Write string to LCD */
unsigned long LcdWriteStr(char* str)
{
    while(*str != '\0')
    {
        LcdWrite((unsigned long)*str++);
    }
    return 0;
}

/* Position LCD cursor assuming 20x4 display */
unsigned long LcdGoTo(unsigned long col, unsigned long row)
{
#if defined(OPT_LCD_ANSI)
    AnsiGoTo(col, row);
#else
    unsigned long addr;

    /* Add row offset (0=first row) */
    switch (row)
    {
        case 0: addr = col;
                break;
        case 1: addr = col + 0x40;
                break;
        case 2: addr = col + 0x14;
                break;
        case 3: addr = col + 0x54;
                break;
        /* Use start address if row is invalid */
        default: addr = 0;
                break;
    }
    addr |= 0x80; /* Set display data address */
    LcdCommand(addr);
#endif
    return 0;
}

/* Write string to LCD at a specified coordinate */
unsigned long LcdWriteAt(unsigned long col, unsigned long row, char* str)
{
    LcdGoTo(col, row);
    return LcdWriteStr(str);
}

/* Clear LCD display */
unsigned long LcdClear(void)
{
#if defined(OPT_LCD_ANSI)
    AnsiClrScr();
    return 0;
#else
    return LcdCommand(0x01);
#endif
}

/* Turn display on or off */
unsigned long LcdDisplay(unsigned long val)
{
    if (val != 0)
        lcdDisplay |= LCD_DISP_ON;
    else
        lcdDisplay &= ~LCD_DISP_ON;

    return LcdCommand(lcdDisplay);
}

/* Turn cursor on or off */
unsigned long LcdCursor(unsigned long val)
{
    if (val != 0)
        lcdDisplay |= (LCD_DISP_CURSOR | LCD_DISP_BLINK);
    else
        lcdDisplay &= ~(LCD_DISP_CURSOR | LCD_DISP_BLINK);

    return LcdCommand(lcdDisplay);
}

/* Turn backlight on or off */
unsigned long LcdBacklight(unsigned long val)
{
    unsigned long prevVal = lcdControl;

    if (val != 0)
        lcdControl |= LCD_BL;
    else
        lcdControl &= ~LCD_BL;
#ifdef LCD_XIO
    XioWrite(LCD_CONTROL, lcdControl);
#elif !defined(OPT_LCD_ANSI)
    GPIOPinWrite(LCD_CONTROL, LCD_CONTROL_MASK, lcdControl);
#endif

    return prevVal;
}

/* Approximate delay for a number of microseconds */
static void delay_us(unsigned long n)
{
    volatile unsigned long i;
    unsigned long iMax = n * 10;


    for (i=0; i<iMax; i++);
}
