--------------------------------------------------------------------------------
-- pc104.vhd : ISA (PC/104) bus master
-- Micromint USA (407)333-4799 www.micromint.com
-- Micromint Support <support@micromint.com>
-- 
-- Version:        2009-07
-- Design Name:    PC/104 Bus Master
-- Module Name:    pc104 - Behavioral 
-- Project Name:   Eagle 100
-- Target Devices: xc9572xl
-- Tool versions:  Xilinx ISE 11.1
--
-- Description: 
--   Implements a PC/104 Bus Master via SPI
--
-- Revision history:
--   2009-07 - 07/31/2009 Upgraded to Xilinx ISE 11.1.
--   2009-01 - 01/27/2009 Refactoring to reduce macrocell usage
--   2008-12 - 12/09/2009 Changed to 9-bit data frames to improve block I/O
--   2008-07 - 07/02/2008 Upgraded to Xilinx ISE 10.1, added additional opcodes
--   2008-04 - 04/09/2008 Initial build using Xilinx ISE 9.2
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.ALL;
use ieee.std_logic_unsigned.ALL;

library work;
use work.micromint_std.all;

entity pc104 is
	port (
		-- Device control
		clk : in std_logic; -- Clock
		reset : in std_logic; -- Reset (RST)
		intr : out std_logic; -- Interrupt
		-- SPI control
		spiTx : in std_logic; -- SPI Master Transmit (SPI0TX)
		spiRx : out std_logic; -- SPI Master Receive (SPI0RX)
		spiClk : in std_logic; -- SPI Clock (SPI0CLK)
		spiSel : in std_logic; -- Chip select (IO_CS)
		-- Ports on extended GPIO connector
		portXA : inout std_logic_vector (7 downto 0); -- Port XA (PXA)
		portXB : inout std_logic_vector (7 downto 0); -- Port XB (PXB)
		portXC : inout std_logic_vector (7 downto 0); -- Port XC (PXC)
		-- LCD interface
		portXD : inout std_logic_vector (7 downto 0); -- Port XD (PXD) LCD Data
		portXE : inout std_logic_vector (3 downto 0); -- Port XE (PXE)
				-- LCD Control: LCD_RS, LCD_RW, LCD_E, LCD_BL
		-- Ports on expansion bus connector
		isaData : inout std_logic_vector (7 downto 0); -- ISA Data (SD7..0)
				-- portXF7..0
		isaAddr : out std_logic_vector (9 downto 0); -- ISA Address (SA9..0)
				-- portXH1..0 portXG7..0
		isaReset : inout std_logic;
		isaAEN : inout std_logic;
		isaBALE : out std_logic;
		isaIOR : inout std_logic;
		isaIOW : inout std_logic;
				-- ISA Control portXH7..4
		-- Ports not used in current implementation
		isaOsc : inout std_logic;
		isaBClk : inout std_logic;
		GCK3 : inout std_logic;
--		sGSR : inout std_logic;
				-- portXH3..2 portXI2..0
		-- User LED
		led2In: in std_logic;	-- PE1/GTS2
		led2Out: out std_logic	-- GCK1
		);
end pc104;

architecture behavioral of pc104 is
	-- Components
	-- ISA Master component
	component isa_master is	   
		port(
			clk, reset : in std_logic;
			AEN, BALE, IOR, IOW: out std_logic;
			enable, RW: in std_logic
		);
	end component; 

	-- Signals
	signal dataRX : std_logic_vector(SPI_WIDTH-1 downto 0);
	signal dataTX : std_logic_vector(SPI_WIDTH-1 downto 0);
	signal cmdPort : std_logic_vector(3 downto 0); -- Command port
	signal cmdCode : std_logic_vector(3 downto 0); -- Command opcode

	signal isaEnable, isaRW : std_logic;
	signal isaDataRX : std_logic_vector (7 downto 0);
	signal isaDataTX : std_logic_vector (7 downto 0);
	signal isaAddrBuf : std_logic_vector (9 downto 0);
	-- Output buffers
	-- Comment out ports that are input only
--	signal bufXA: std_logic_vector(7 downto 0);
--	signal bufXB: std_logic_vector(7 downto 0);
--	signal bufXC: std_logic_vector(7 downto 0);
--	signal bufXD: std_logic_vector(7 downto 0);
--	signal bufXE: std_logic_vector(3 downto 0);
	-- Output enable
--	signal oeXA: std_logic := '0';
--	signal oeXB: std_logic := '0';
--	signal oeXC: std_logic := '0';
--	signal oeXD: std_logic := '0';
--	signal oeXE: std_logic := '0';
begin
	-- User LED
	led2Out <= led2In;

	-- Enable outputs
	-- Comment out ports that are input only
--	portXA <= bufXA when oeXA = '1' else (others => 'Z');
--	portXB <= bufXB when oeXB = '1' else (others => 'Z');
--	portXC <= bufXC when oeXC = '1' else (others => 'Z');
--	portXD <= bufXD when oeXD = '1' else (others => 'Z');
--	portXE <= bufXE when oeXE = '1' else (others => 'Z');

	-- ISA bus logic
	isaReset <= not reset;
	isaAddr <= isaAddrBuf when isaAEN = '1' else
					(others => 'Z');
	isaData <= isaDataTx when isaIOW = '0' else -- Store data to slave
					(others => 'Z');
	isaBClk <= isaEnable;

	-- Object instances for SPI slave and ISA master
SPI0 : spi_slave port map (clk=>spiClk, sel=>spiSel,
							mosi=>spiTx, miso=>spiRx, reset=>reset,
							dataRX=>dataRX, dataTX=>dataTX);

ISA0 : isa_master port map (clk=>clk, reset=>isaReset,
							AEN=>isaAEN, BALE=>isaBALE, IOR=>isaIOR, IOW=>isaIOW,
							enable=>isaEnable, RW=>isaRW);

	-- Sequential logic
	-- Command handler
	cmd_process : process(reset, spiSel)
	begin
		-- Device reset?
		if (reset = '0') then
			-- Tristate ports after a reset
			portXA <= (others => 'Z');
			portXB <= (others => 'Z');
			portXC <= (others => 'Z');
			portXD <= (others => 'Z');
			portXE <= (others => 'Z');
--			oeXA <= '0';
--			oeXB <= '0';
--			oeXC <= '0';
--			oeXD <= '0';
--			oeXE <= '0';
--			regXA <= (others => '0');
--			regXB <= (others => '0');
--			regXC <= (others => '0');
--			regXD <= (others => '0');
			-- Initialize ISA signals
			isaEnable <= '0'; isaRW <= '0';
		-- Device deselected? If so, a frame was just received
		elsif rising_edge(spiSel) then
			-- If MSB=1 frame contains a command
			if (dataRX(SPI_WIDTH-1) = '1') then
				cmdCode <= dataRX(7 downto 4);
				cmdPort <= dataRX(3 downto 0);
				-- OpCode 0001 = Read data from port
				if (dataRX(4) = '1') then
					-- Place port data on SPI transmit register
					case dataRX(3 downto 0) is
						when "0001" => dataTX(7 downto 0) <= portXA;
						when "0010" => dataTX(7 downto 0) <= portXB;
						when "0011" => dataTX(7 downto 0) <= portXC;
						when "0100" => dataTX(7 downto 0) <= portXD;
						when "0101" => dataTX(7 downto 0) <= "0000" & portXE;
						when "0110" =>
								dataTX(7 downto 0) <= isaDataRX; -- ISA Read data
						when "0111" => dataTX(7 downto 0) <= isaAddrBuf(7 downto 0);
						when "1000" => dataTX(7 downto 0) <= "000000" & isaAddrBuf(9 downto 8);
						when "1001" => -- ISA Latch data
								isaEnable <= '1'; isaRW <= '0';
						when "1010" => -- ISA Status
								dataTX(7 downto 0) <= "0000000" & isaAEN;
								isaEnable <= '0'; isaRW <= '0';
--						when "1011" => dataTX(7 downto 0) <= regXA;
--						when "1100" => dataTX(7 downto 0) <= regXB;
--						when "1101" => dataTX(7 downto 0) <= regXC;
--						when "1110" => dataTX(7 downto 0) <= regXD;
						when others => null;
					end case;
				end if;
			-- Else, frame is data
			-- OpCode 0010 = Write data to port
			elsif (cmdCode(1) = '1') then
				-- Update port data
				-- Comment out ports that are input only
				case cmdPort is
--					when "0001" => bufXA <= dataRX(7 downto 0); oeXA <= '1';
--					when "0010" => bufXB <= dataRX(7 downto 0); oeXB <= '1';
--					when "0011" => bufXC <= dataRX(7 downto 0); oeXC <= '1';
--					when "0100" => bufXD <= dataRX(7 downto 0); oeXD <= '1';
--					when "0101" => bufXE <= dataRX(3 downto 0); oeXE <= '1';
					when "0110" =>
							isaDataTX <= dataRX(7 downto 0); -- ISA write data
							isaEnable <= '1'; isaRW <= '1';
					when "0111" => isaAddrBuf(7 downto 0) <= dataRX(7 downto 0);
					when "1000" => isaAddrBuf(9 downto 8) <= dataRX(1 downto 0);
--					when "1011" => regXA <= dataRX(7 downto 0);
--					when "1100" => regXB <= dataRX(7 downto 0);
--					when "1101" => regXC <= dataRX(7 downto 0);
--					when "1110" => regXD <= dataRX(7 downto 0);
					when others => null;
				end case;
			end if;
		end if;
	end process cmd_process;

	-- Load handler
	load_process : process(isaIOR)
	begin
		if rising_edge(isaIOR) then
			-- Load data from slave
			isaDataRX <= isaData;
		end if;
	end process load_process;

end behavioral;
