--------------------------------------------------------------------------------
-- spi_slave.vhd : Simple SPI slave
-- Micromint USA (407)333-4799 www.micromint.com
-- Micromint Support <support@micromint.com>
-- 
-- Version:        2009-07
-- Design Name:    SPI Slave
-- Module Name:    spi_slave - Behavioral 
-- Project Name:   Eagle 100
-- Target Devices: xc9572xl
-- Tool versions:  Xilinx ISE 11.1
--
-- Description: 
--   Implements a simple SPI slave with receive and transmit capabilities.
--   Uses Freescale SPI frame format, polarity CPOL=0, phase CPHA=0.
--   Data is captured on rising edge, propagated on falling edge of SPI clock.
--   No overflow or timeout checks. Error checks should be done by SPI master.
--
-- Revision history:
--   2009-07 - 07/31/2009 Upgraded to Xilinx ISE 11.1.
--   2009-01 - 01/27/2009 Refactoring to reduce macrocell usage
--   2008-12 - 12/09/2009 Changed to 9-bit data frames to improve block I/O
--   2008-07 - 07/02/2008 Upgraded to Xilinx ISE 10.1
--   2008-04 - 04/09/2008 Initial build using Xilinx ISE 9.2
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.ALL;
use ieee.std_logic_unsigned.ALL;

library work;
use work.micromint_std.all;

entity spi_slave is
	port (
		-- SPI Signals 
		clk : in  std_logic; -- SPI Clock (SCLK)
		sel : in  std_logic; -- Slave Select (SS) 0=selected
		mosi : in  std_logic; -- Master Out Slave In (MOSI)
		miso : out std_logic; -- Master In Slave Out (MISO)
		-- Component interface
		reset : in  std_logic; -- Reset
		dataRX : out std_logic_vector(SPI_WIDTH-1 downto 0); -- Receive register
		dataTX : in  std_logic_vector(SPI_WIDTH-1 downto 0) -- Transmit register
		);
end spi_slave;

architecture behavioral of spi_slave is
	signal count : integer range 0 to SPI_WIDTH-1;
begin
	-- Combinatorial logic
	-- Copy SPI slave data from transmit register to output port
	miso <= dataTX(count) when sel = '0' else
				'Z'; -- Tristate output if not selected to allow multiple slaves

	-- Sequential logic
	-- Receive process
	rx_process : process(clk)
	begin
		if rising_edge(clk) then
			-- On rising edge shift input data from SPI master to receive register
			dataRX(count) <= mosi;
		end if;
	end process rx_process;

	-- Counter process
	cnt_process : process(reset, clk, sel)
	begin
		-- Device reset or deselected?
		if (reset = '0' or sel = '1') then
			-- If so, reset counter to start a new frame
			count <= SPI_WIDTH-1;
		elsif falling_edge(clk) then
			-- If data left, on falling edge decrement counter to shift output
			if (count /= 0) then
				count <= count - 1;
			end if;
		end if;
	end process cnt_process;
end behavioral;
