--------------------------------------------------------------------------------
-- xtender.vhd : SPI port extender
-- Micromint USA (407)333-4799 www.micromint.com
-- Micromint Support <support@micromint.com>
-- 
-- Version:        2009-07
-- Design Name:    SPI Port Extender
-- Module Name:    xtender - Behavioral 
-- Project Name:   Eagle 100
-- Target Devices: xc9572xl
-- Tool versions:  Xilinx ISE 11.1
--
-- Description: 
--   Implements an SPI port extender.
--
-- Revision history:
--   2009-07 - 07/31/2009 Upgraded to Xilinx ISE 11.1.
--   2009-01 - 01/27/2009 Refactoring to reduce macrocell usage
--   2008-12 - 12/09/2009 Changed to 9-bit data frames to improve block I/O
--   2008-07 - 07/02/2008 Upgraded to Xilinx ISE 10.1, added additional opcodes
--   2008-04 - 04/09/2008 Initial build using Xilinx ISE 9.2
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.ALL;
use ieee.std_logic_unsigned.ALL;

library work;
use work.micromint_std.all;

entity xtender is
	port (
		-- Device control
		clk : in std_logic; -- Clock
		reset : in std_logic; -- Reset (RST)
		intr : out std_logic; -- Interrupt
		-- SPI control
		spiTx : in std_logic; -- SPI Master Transmit (SPI0TX)
		spiRx : out std_logic; -- SPI Master Receive (SPI0RX)
		spiClk : in std_logic; -- SPI Clock (SPI0CLK)
		spiSel : in std_logic; -- Chip select (IO_CS)
		-- Ports on extended GPIO connector
		portXA : inout std_logic_vector (7 downto 0); -- Port XA (PXA)
		portXB : inout std_logic_vector (7 downto 0); -- Port XB (PXB)
		portXC : inout std_logic_vector (7 downto 0); -- Port XC (PXC)
		-- LCD interface
		portXD : inout std_logic_vector (7 downto 0); -- Port XD (PXD) LCD Data
		portXE : inout std_logic_vector (3 downto 0); -- Port XE (PXE)
				-- LCD Control: LCD_RS, LCD_RW, LCD_E, LCD_BL
		-- Ports on expansion bus connector
		portXF : inout std_logic_vector (7 downto 0); -- Port XF (PXF)
				-- ISA Data SD7..0
		portXG : inout std_logic_vector (7 downto 0); -- Port XG (PXG)
				-- ISA Address SA7..0
		portXH : inout std_logic_vector (7 downto 0); -- Port XH (PXH)
				-- ISA Control: PC104_RST, IOW, IOR, AEN, ISA_BALE, ISA_OSC, SA9..8
		portXI : inout std_logic_vector (2 downto 0); -- Port XI (PXI)
				-- ISA_BCLK, GCK3, GSR
		-- User LED
		led2In: in std_logic;	-- PE1/GTS2
		led2Out: out std_logic	-- GCK1
		);
end xtender;

architecture behavioral of xtender is
	-- Signals
	signal dataRX : std_logic_vector(SPI_WIDTH-1 downto 0);
	signal dataTX : std_logic_vector(SPI_WIDTH-1 downto 0);
	signal cmdPort : std_logic_vector(3 downto 0); -- Command port
	signal cmdCode : std_logic_vector(3 downto 0); -- Command opcode
	-- Output buffers
	-- Comment out ports that are input only
	signal bufXA: std_logic_vector(7 downto 0);
	signal bufXB: std_logic_vector(7 downto 0);
--	signal bufXC: std_logic_vector(7 downto 0);
	signal bufXD: std_logic_vector(7 downto 0);
	signal bufXE: std_logic_vector(3 downto 0);
--	signal bufXF: std_logic_vector(7 downto 0);
--	signal bufXG: std_logic_vector(7 downto 0);
--	signal bufXH: std_logic_vector(7 downto 0);
--	signal bufXI: std_logic_vector(7 downto 0);
	-- Output enable
	signal oeXA: std_logic := '0';
	signal oeXB: std_logic := '0';
--	signal oeXC: std_logic := '0';
	signal oeXD: std_logic := '0';
	signal oeXE: std_logic := '0';
--	signal oeXF: std_logic := '0';
--	signal oeXG: std_logic := '0';
--	signal oeXH: std_logic := '0';
--	signal oeXI: std_logic := '0';
begin
	-- User LED
	led2Out <= led2In;

	-- Enable outputs
	-- Comment out ports that are input only
	portXA <= bufXA when oeXA = '1' else (others => 'Z');
	portXB <= bufXB when oeXB = '1' else (others => 'Z');
--	portXC <= bufXC when oeXC = '1' else (others => 'Z');
	portXD <= bufXD when oeXD = '1' else (others => 'Z');
	portXE <= bufXE when oeXE = '1' else (others => 'Z');
--	portXF <= bufXF when oeXF = '1' else (others => 'Z');
--	portXG <= bufXG when oeXG = '1' else (others => 'Z');
--	portXH <= bufXH when oeXH = '1' else (others => 'Z');
--	portXI <= bufXI when oeXI = '1' else (others => 'Z');

	-- Object instance for SPI slave
SPI0 : spi_slave port map (clk=>spiClk, sel=>spiSel,
							mosi=>spiTx, miso=>spiRx, reset=>reset,
							dataRX=>dataRX, dataTX=>dataTX);

	-- Sequential logic
	-- Command handler
	cmd_process : process(reset, spiSel)
	begin
		-- Device reset?
		if (reset = '0') then
			-- Tristate ports after a reset
			portXA <= (others => 'Z');
			portXB <= (others => 'Z');
			portXC <= (others => 'Z');
			portXD <= (others => 'Z');
			portXE <= (others => 'Z');
			portXF <= (others => 'Z');
			portXG <= (others => 'Z');
			portXH <= (others => 'Z');
			portXI <= (others => 'Z');
			oeXA <= '0';
			oeXB <= '0';
--			oeXC <= '0';
			oeXD <= '0';
			oeXE <= '0';
--			oeXF <= '0';
--			oeXG <= '0';
--			oeXH <= '0';
--			oeXI <= '0';
--			regXA <= (others => '0');
--			regXB <= (others => '0');
--			regXC <= (others => '0');
--			regXD <= (others => '0');
		-- Device deselected? If so, a frame was just received
		elsif rising_edge(spiSel) then
			-- If MSB=1 frame contains a command
			if (dataRX(SPI_WIDTH-1) = '1') then
				cmdCode <= dataRX(7 downto 4);
				cmdPort <= dataRX(3 downto 0);
				-- OpCode 0001 = Read data from port
				if (dataRX(4) = '1') then
					-- Place port data on SPI transmit register
					case dataRX(3 downto 0) is
						when "0001" => dataTX(7 downto 0) <= portXA;
						when "0010" => dataTX(7 downto 0) <= portXB;
						when "0011" => dataTX(7 downto 0) <= portXC;
						when "0100" => dataTX(7 downto 0) <= portXD;
						when "0101" => dataTX(7 downto 0) <= "0000" & portXE;
						when "0110" => dataTX(7 downto 0) <= portXF;
						when "0111" => dataTX(7 downto 0) <= portXG;
						when "1000" => dataTX(7 downto 0) <= portXH;
						when "1001" => dataTX(7 downto 0) <= "00000" & portXI;
--						when "1011" => dataTX(7 downto 0) <= regXA;
--						when "1100" => dataTX(7 downto 0) <= regXB;
--						when "1101" => dataTX(7 downto 0) <= regXC;
--						when "1110" => dataTX(7 downto 0) <= regXD;
						when others => null;
					end case;
				end if;
			-- Else, frame is data
			-- OpCode 0010 = Write data to port
			elsif (cmdCode(1) = '1') then
				-- Update port data
				-- Comment out ports that are input only
				case cmdPort is
					when "0001" => bufXA <= dataRX(7 downto 0); oeXA <= '1';
					when "0010" => bufXB <= dataRX(7 downto 0); oeXB <= '1';
--					when "0011" => bufXC <= dataRX(7 downto 0); oeXC <= '1';
					when "0100" => bufXD <= dataRX(7 downto 0); oeXD <= '1';
					when "0101" => bufXE <= dataRX(3 downto 0); oeXE <= '1';
--					when "0110" => bufXF <= dataRX(7 downto 0); oeXF <= '1';
--					when "0111" => bufXG <= dataRX(7 downto 0); oeXG <= '1';
--					when "1000" => bufXH <= dataRX(7 downto 0); oeXG <= '1';
--					when "1001" => bufXI <= dataRX(2 downto 0); oeXI <= '1';
--					when "1011" => regXA <= dataRX(7 downto 0);
--					when "1100" => regXB <= dataRX(7 downto 0);
--					when "1101" => regXC <= dataRX(7 downto 0);
--					when "1110" => regXD <= dataRX(7 downto 0);
					when others => null;
				end case;
			end if;
		end if;
	end process cmd_process;

end behavioral;
